
;;; test "Gloss" API

;FIXME use "variable" vs "parameter"

(module (test-utils gloss support)

(;export
  ;Parameters
  test-indent-width
  test-first-indentation
  test-max-indentation
  test-indentation-char
  ;
  test-group-level
  test-group-indent-width
  test-group-indent-string)

(import scheme ;utf8
  (chicken base)
  (chicken syntax)
  (only (chicken process-context) get-environment-variable))

(define get-environment-variable/default
  (case-lambda
    ((nm)
      (get-environment-variable/default nm #f))
    ((nm def)
      (cond ((get-environment-variable nm) => string->number)
            (else def))) ) )

;moremacros

(import-for-syntax (only (chicken base) symbol-append))

(define-syntax checked-guard
  (er-macro-transformer
    (lambda (frm rnm cmp)
      (##sys#check-syntax 'checked-guard frm '(_ symbol symbol . _))
      (let ((?locnam (cadr frm))
            (?typnam (caddr frm))
            (?body (cdddr frm))
        (_lambda (rnm 'lambda)) )
        `(,_lambda (obj)
          (,(symbol-append 'check- (strip-syntax ?typnam)) ',?locnam obj)
          ,@?body
          obj ) ) ) ) )

; not too proud of the name `variable' but ...

(define (make-variable init #!optional (guard identity))
  (let ((value (guard init)))
    (define (setter obj) (set! value (guard obj)))
    (getter-with-setter
      ;ugly but like parameter
      (lambda args
        (if (null? args) value
            (let ((new (car args)))
              (setter new)
              new ) ) )
      ;emphasize not a paramter
      setter) ) )

(define-syntax define-variable
  (syntax-rules ()
    ((_ ?name ?init) (define ?name (make-variable ?init)) )
    ((_ ?name ?init ?guard) (define ?name (make-variable ?init ?guard)) ) ) )

(define-syntax define-checked-variable
  (syntax-rules ()
    ((_ ?name ?init ?typnam ?body0 ...)
      (define-variable ?name ?init (checked-guard ?name ?typnam ?body0 ...)) ) ) )

;(check-errors sys)

(define (check-char loc obj) (##sys#check-char obj loc) obj)
(define (check-exact-unsigned-integer loc obj) (##sys#check-exact-uinteger obj loc) obj)

;;

(define (check-indentation-amount loc obj) (check-exact-unsigned-integer loc obj))

(define-checked-variable test-indent-width
  (get-environment-variable/default "TEST_INDENT_WIDTH" 4)
  indentation-amount)

(define-checked-variable test-first-indentation
  (get-environment-variable/default "TEST_FIRST_INDENTATION" 1)
  indentation-amount)

(define-checked-variable test-max-indentation
  (get-environment-variable/default "TEST_MAX_INDENTATION" 5)
  indentation-amount)

(define-checked-variable test-indentation-char
  (string-ref (get-environment-variable/default "TEST_INDENTATION_CHAR" " ") 0)
  char)

;;

;test?

(define (test-group-ref group field . o)
  (define (assq-ref ls key . o)
    (cond ((assq key ls) => cdr)
          ((pair? o) (car o))
          (else #f)) )
  (apply assq-ref (cdr group) field o) )

;;

(define (*test-group-level group)
  (add1 (- (test-group-ref group 'level 0) (test-first-indentation))) )

(define (test-group-level group)
  (min (test-max-indentation) (max 0 (*test-group-level group))) )

(define (test-group-indent-width group)
  (* (test-indent-width) (test-group-level group)) )

(define (test-group-indent-string group)
  (if (not group) ""
      (make-string (test-group-indent-width group) (test-indentation-char))) )

) ;module (test-utils gloss support)

;;

(module (test-utils gloss basic)

(;export
  glossln
  (glossn display-gloss-marker)
  gloss)

(import scheme ;utf8
  (chicken base)
  (chicken syntax)
  test
  (test-utils gloss support))

;;

(define-constant TEST-GLOSS-MARKER "-->")

(define (display-gloss-marker)
  (display (test-group-indent-string (current-test-group)))
  (display TEST-GLOSS-MARKER)
  (display #\space) )

;;

(define-syntax glossln
  (syntax-rules ()
    ((glossln)
      (begin (newline) (flush-output) ) ) ) )

(define-syntax glossn
  (syntax-rules ()
    ((glossn)
      (begin) )
    ((glossn ?obj)
      (begin (display-gloss-marker) (display ?obj)))
    ((glossn ?obj ...)
      (begin
        (display-gloss-marker)
        (for-each (lambda (x) (display x) (display #\space)) (list ?obj ...))) ) ) )

(define-syntax gloss
  (syntax-rules ()
    ((gloss)
      (glossln) )
    ((gloss ?obj ...)
      (begin (glossn ?obj ...) (glossln)) ) ) )

) ;module (test-utils gloss basic)

;; Formatted Gloss

;Needs a format, builtin or egg
;(import (test gloss format) (only (chicken format) format))
;(import (test gloss format) format)

(module (test-utils gloss format)

(;export
  glossnf
  glossf)

(import scheme ;utf8
  (chicken base)
  (chicken syntax)
  test
  (test-utils gloss basic))

(define-syntax glossnf
  (syntax-rules ()
    ((glossnf ?fmt ?arg0 ...)
      (glossn (format #f ?fmt ?arg0 ...)) ) ) )

(define-syntax glossf
  (syntax-rules ()
    ((glossf ?fmt ?arg0 ...)
      (begin (glossnf ?fmt ?arg0 ...) (glossln) ) ) ) )

) ;module (test-utils gloss format)

(module (test-utils gloss) ()

(import scheme (chicken module))

(import (test-utils gloss support) (test-utils gloss basic) (test-utils gloss format))
(reexport (test-utils gloss support) (test-utils gloss basic) (test-utils gloss format))
) ;(test-utils gloss)
